#pragma strict

var box : GameObject;

static var spacing = 40;
static var eaten = new Hashtable();
// static var eaten = {};
// JavaScript object literal syntax works too. It creates a Boo.Lang.Hash()
// Boo.Lang.Hash is Boo's hash class, a subclass of Hashtable 
static var map_edge = new Vector2();
static var map_dim = new Vector2();

private var sdim = new Vector2();
private var myCamera : Camera;

function grid_snap_floor(a : float) {
  return Mathf.Floor(a / spacing) * spacing;
}

function draw_map(startx : int, starty : int, old_startx : int, old_starty : int, width : int, height : int) {
  var i : int;
  var j : int;
  for (i = startx; i < startx + width; i += spacing) {
    for (j = starty; j < starty + height; j += spacing) {
      if ((i < old_startx || i > old_startx + width - spacing ||
          j < old_starty || j > old_starty + height - spacing) &&
          (!eaten.ContainsKey(i) || !(eaten[i] as Hashtable).ContainsKey(j))) { 
        // According to the docs Instantiate() is FAST 
        Instantiate(box, Vector3(i, j, 2), Quaternion.identity);
      }
    }
  }
}

function Start () {
  var bot_right : Vector3;
  var top_left : Vector3;

  // cache camera component for better performance
  myCamera = camera;

  bot_right = myCamera.ViewportToWorldPoint(Vector3(0, 0, 0));
  top_left = myCamera.ViewportToWorldPoint(Vector3(1, 1, 0));

  sdim.x = top_left.x - bot_right.x;
  sdim.y = top_left.y - bot_right.y;

  map_edge.x = grid_snap_floor(bot_right.x) - spacing;
  map_edge.y = grid_snap_floor(bot_right.y) - spacing;

  map_dim.x = grid_snap_floor(top_left.x) + spacing * 2 - map_edge.x;
  map_dim.y = grid_snap_floor(top_left.y) + spacing * 2 - map_edge.y;

  draw_map(map_edge.x, map_edge.y,
    map_edge.x - map_dim.x, map_edge.y - map_dim.y,
    map_dim.x, map_dim.y);
}

function Update () {
  check_bounds();
}

function check_bounds() {
  var old_edge = new Vector2();
  var cpos = myCamera.ViewportToWorldPoint(Vector3(0, 0, 2));

  if ((cpos.y < map_edge.y + spacing) ||
     (cpos.y + sdim.y > map_edge.y + map_dim.y - spacing) ||
     (cpos.x < map_edge.x + spacing) ||
     (cpos.x + sdim.x > map_edge.x + map_dim.x - spacing))  {

    old_edge.x = map_edge.x;
    old_edge.y = map_edge.y;
    map_edge.x = grid_snap_floor(cpos.x) - spacing;
    map_edge.y = grid_snap_floor(cpos.y) - spacing;

    draw_map(map_edge.x, map_edge.y, old_edge.x, old_edge.y, map_dim.x, map_dim.y);
  }
}
